@@ -17,7 +17,7 @@ use HTML::DOM::Node 'DOCUMENT_NODE';
 use Scalar::Util 'weaken';
 use URI;
 
-our $VERSION = '0.054';
+our $VERSION = '0.055';
 our @ISA = 'HTML::DOM::Node';
 
 require    HTML::DOM::Collection;
@@ -45,7 +45,7 @@ HTML::DOM - A Perl implementation of the HTML Document Object Model
 
 =head1 VERSION
 
-Version 0.054 (alpha)
+Version 0.055 (alpha)
 
 B<WARNING:> This module is still at an experimental stage.  The API is 
 subject to change without
@@ -1045,16 +1045,20 @@ sub URL {
 sub body { # ~~~ this needs to return the outermost frameset element if
             #     there is one (if the frameset is always the second child
             #     of <html>, then it already does).
+	my $body = ($_[0]->documentElement->content_list)[1];
+	if (!$body || $body->tag !~ /^(?:body|frameset)\z/) {
+		$body = $_[0]->find('body','frameset');
+	}
 	if(@_>1) {
 		my $doc_elem = $_[0]->documentElement;
 		# I'm using the replaceChild rather than replace_with,
 		# despite the former's convoluted syntax, since the former
 		# has the appropriate error-checking code (or will), and
 		# also because it triggers mutation events.
-		$doc_elem->replaceChild($_[1],($doc_elem->content_list)[1])
+		$doc_elem->replaceChild($_[1],$body)
 	}
 	else {
-		($_[0]->documentElement->content_list)[1];
+		$body
 	}
 }
 
@@ -1208,8 +1212,9 @@ sub cookie {
 
 =item getElementsByClassName
 
-These two do what their names imply.  The latter will return a list in list
-context, or a node list object in scalar context.  Calling it in list
+These three do what their names imply.  The last two
+will return a list in list context, or a node list
+object in scalar context.  Calling them in list
 context is probably more efficient.
 
 =cut
@@ -1242,30 +1247,8 @@ sub getElementsByName {
 }
 
 sub getElementsByClassName {
-	my($self,$names) = @_;
-
-	my $cref;
-	if(defined $names) {
-	 no warnings 'uninitialized';
- 	 $names
-	  = join ".*", map "\\b$_\\b", sort split /[ \t\n\f\r]+/, $names;
-	 $cref = sub {
-	  join(" ", sort split /[ \t\n\f\r]+/, $_[0]->attr('class'))
-	   =~ $names
-	 };
-	}
-	else { $cref = sub {} }
-
-	if (wantarray) {
-		return $self->look_down($cref);
-	}
-	else {
-		my $list = HTML::DOM::NodeList::Magic->new(
-			  sub { $self->look_down($cref); }
-		);
-		$self-> _register_magic_node_list($list);
-		$list;
-	}
+	splice @_, 2, @_, 1; # Remove extra elements; add a true third elem
+	goto &HTML'DOM'Element'_getElementsByClassName;
 }
 
 # ---------- DocumentEvent interface -------------- #
